==> tslm-1.1.0.2-Server-img.tar.gz: opt/IBM/ermm/lib/cli/TsmDsmadmc.pm <==
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2012 All Rights Reserved
#-----------------------------------------------------------------------
#
# @version 1.3 - 12/01/26 01:04:56
#
# @brief Perl interface to dsmadmc
#
# This module wraps the calls to dsmadmc for scripts that want
# to use it as interface to access a TSM server.
#
# Change activities:
#     - 2004-10-21: Initial version
#     - 2005-12-20: Improved error handling
#
#-----------------------------------------------------------------------

package TsmDsmadmc;

use lib '.';

use ErmmTool;

sub new {
    my ($class, $dsmadmc,  $libname, $dsmserver, $id, $password, $debug) = @_;
    my $self = {
        '_dsmadmc'  => $dsmadmc,
        '_libname'  => $libname,
        '_server'   => $dsmserver,
        '_id'       => $id,
        '_password' => $password,
        '_debug'    => $debug
    };
    bless ($self, $class);
    return $self;
}

# Get all volumes for a certain library. The logic is to
#  1) find all device classes that reference this library
#  2) for each device class get all volumes.
# This is necessary, because
#  - there could be different device types
#  - a setup with storage agents requires one devclass per node
# Return 0 on success and -1 on error.
sub volumes {
  my ($self, $volumes) = @_;

  my @devclasses;
  if ($self->devclasses(\@devclasses)) {
      $$volumes = {};
      return -1;
  };

  if($self->{_debug}) {
      foreach(@devclasses) {
          ErmmTool::printtext('SYNCMSG1015', $_);
      }
  }

  DEVCLASSES: foreach my $devclass (@devclasses) {

      $self->_dsmcmd("query volume devclass=$devclass");
      foreach my $line ( @{$self->{_rawmsg}} ) {
          if ($line =~ /ANR2034E/) {
              # There are currently no volumes mounted according to TSM.
              next DEVCLASSES;

          } elsif ($line =~ /AN\w\d{4}\w .+/) {
              $$volumes = {};
              return -1;
          }

          my ($volume, $stgpool, $devclass,
              $capacity, $utilization) = split /,/, $line;
          $$volumes->{$volume} = { 'Type'        => 'volume',
                                   'StoragePool' => $stgpool,
                                   'Capacity'    => $capacity,
                                   'Utilization' => $utilization }
      }
  }

  return 0;
}

# Get all device classes that reference the library and return the
# devclasses names as list
# Return 0 on success and -1 on error.
sub devclasses {
    my($self, $devclasses) = @_;

    $self->_dsmcmd("query devclass format=detailed");
    foreach my $line (@{$self->{_rawmsg}}) {
          if ($line =~ /AN\w\d{4}\w .+/) {
              $$volumes = {};
              return -1;
          }
        my ($devclass, $strategy, $count, $type, $format, $capacity, $limit, $wait, $retention,
            $prefix, $devclass_library, $directory, $server, $retry) = split /,/, $line;

        if(defined $devclass_library && $devclass_library ne '') {
            if($self->{_debug}) {
                ErmmTool::printtext('SYNCMSG1016', $devclass, $devclass_library);
            }

            if ($self->{_libname} eq $devclass_library) {
                push @$devclasses, $devclass;
            }
        }
    }

    if (@$devclasses == 0) {
        (@{$self->{_rawmsg}}) = "Couldn't find TSM library $self->{_libname}.";
        return -1;
    }
    return 0;
}

# Get all drmedia (e.g. database backups) for one library.
#
# Since there is no limit option for the TSM command, this
# function gets all and then filters the result for the
# drmedia in the library.
#
# This should be ok, because there should not be much drmedias.
# Return 0 on success and -1 on error.
# We use the volume history as this won't start the ELM (unlike 'q drmedia')
sub drmedia {
    my ($self, $volumes) = @_;
    $self->_dsmcmd("query volhistory type=dbbackup");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if ($line =~ /ANR2034E/) {
            # There are currently no drmedia according to TSM.
            $$volumes = {};
            return 0;
        } elsif ($line =~ /AN\w\d{4}\w .+/) {
            $$volumes = {};
            return -1;
        # volume type, device class and volume name are needed
        } elsif ($line =~ /.*,(.+),.*,.*,.*,(.+),(.+),.*,.*/) {
            my $state = $1;
            my $deviceClass = $2;
            my $volume = $3;

            # We need to get the library of the volume (and we know
            # the volume exists). The match from volume to library is
            # done using the device class. A device class can only
            # have one library so use scalar.
            $self->_dsmcmd("query devc $deviceClass f=d");
            my ($devc) = @{$self->{_rawmsg}};
            my @devClassInfo = split /,/, $devc;
            my $library = $devClassInfo[10] if defined $devClassInfo[10];

            if ($self->{_libname} eq $library) {
                $$volumes->{$volume} = { 'Type'       => 'drmedia',
                                         'State'      => $state,
                                         'LastUpdate' => $lastupdate };
            }
        }
    }

    return 0;
} # end sub drmedia

# Get all snapshot volumes from the TSM server
# Return 0 on success and -1 on error
# We use the volume history as this won't start the ELM (unlike a SELECT ...)
sub snapshot {
    my ($self, $volumes) = @_;
    $self->_dsmcmd("query volhistory type=dbsnapshot");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if ($line =~ /ANR2034E/) {
            # There are currently no snapshot volumes according to TSM.
            $$volumes = {};
            return 0;
        } elsif ($line =~ /AN\w\d{4}\w .+/) {
            $$volumes = {};
            return -1;
        # volume type, device class and volume name are needed
        } elsif ($line =~ /.*,(.+),.*,.*,.*,(.+),(.+),.*,.*/) {
            my $state = $1;
            my $deviceClass = $2;
            my $volume = $3;

            # We need to get the library of the volume (and we know
            # the volume exists). The match from volume to library is
            # done using the device class. A device class can only
            # have one library so use scalar.
            $self->_dsmcmd("query devc $deviceClass f=d");
            my ($devc) = @{$self->{_rawmsg}};
            my @devClassInfo = split /,/, $devc;
            my $library = $devClassInfo[10] if defined $devClassInfo[10];

            if ($self->{_libname} eq $library) {
                $$volumes->{$volume} = { 'Type'       => 'dbsnapshot',
                                         'State'      => $state,
                                         'LastUpdate' => $lastupdate };
            }
        }
    }

    return 0;
} # end sub snapshot

# Get all current mounts from the TSM server.
# Return 0 on success and -1 on error.
sub mounts {
  my ($self, $volumes) = @_;

  # There is no nice CSV format for this command, so the plan is to retrieve
  # the mounted volumes from  messages that look like this:
  # ANR8330I LTO volume 483AFQ is mounted R/W in drive TSLM
  $self->_dsmcmd("query mount");
  foreach my $line ( @{$self->{_rawmsg}} ) {

    if(defined && /^ANR8330I.*volume ([^ ]+) .*drive ([^ ]+) /) {
      my $volume  = $1;
      my $library = $2;

      if($library eq $self->{_libname} ) {
        $$volumes->{$volume} = { };
      }
    } elsif ($line =~ /ANR2034E/) {
        # There are currently no volumes mounted according to TSM.
       $$volumes = {};
        return 0;
    } elsif ($line =~ /AN\w\d{4}\w .+/) {
       $$volumes = {};
       return -1;
    }
  }

  return 0;
}

# Get last response.
sub getRawMsg {
    my ($self) = @_;
    return @{$self->{_rawmsg}};
}

# Issue TSM command using dsmadmc
sub _dsmcmd {
    my ($self, $dsmcmd) = @_;

    my $dsmflags='-commadelimited -dataonly=yes';

    my $command
        = $self->{_dsmadmc} . ' ' . $dsmflags
        . ' -server=' . $self->{_server}
        . ' -id=' . $self->{_id}
        . ' -password=' . $self->{_password}
        . ' ' . $dsmcmd;

  if($self->{_debug}) {
    ErmmTool::printtext('SYNCMSG1011', $command);
  }

  open my $query, "$command 2>&1 |" or die ErmmTool::printtext('SYNCMSG1014');
  my @result;
  while (<$query>) {
    chomp;
    push @result, $_;
    if($self->{_debug}) {
        ErmmTool::printtext('SYNCMSG1013', $_);
    }
  }
  close $query;
  $self->{_rawmsg} = \@result;
}

1;

==> tslm-1.2.0.1-Server-img.tar.gz: opt/IBM/ermm/lib/cli/TsmDsmadmc.pm <==
#-----------------------------------------------------------------------
# Licensed Materials - Property of IBM
#
# IBM Tape System Library Manager
#
# (C) Copyright IBM Corp. 2004, 2014 All Rights Reserved
#-----------------------------------------------------------------------
#
#
# @version 1.5 - 14/11/04 22:27:08
#
# @brief Perl interface to dsmadmc
#
# This module wraps the calls to dsmadmc for scripts that want
# to use it as interface to access a TSM server.
#
# Change activities:
#     - 2004-10-21: Initial version
#     - 2005-12-20: Improved error handling
#     - 2014-07-18: added backkupset and export (74181)
#     - 2014-11-04: fixed reconcile process (76755)
#
#-----------------------------------------------------------------------

package TsmDsmadmc;

use lib '.';

use ErmmTool;

sub new {
    my ($class, $dsmadmc,  $libname, $dsmserver, $id, $password, $debug) = @_;
    my $self = {
        '_dsmadmc'  => $dsmadmc,
        '_libname'  => $libname,
        '_server'   => $dsmserver,
        '_id'       => $id,
        '_password' => $password,
        '_debug'    => $debug
    };
    bless ($self, $class);
    return $self;
}

# Get all volumes for a certain library. The logic is to
#  1) find all device classes that reference this library
#  2) for each device class get all volumes.
# This is necessary, because
#  - there could be different device types
#  - a setup with storage agents requires one devclass per node
# Return 0 on success and -1 on error.
sub volumes {
  my ($self, $volumes) = @_;

  my @devclasses;
  if ($self->devclasses(\@devclasses)) {
      $$volumes = {};
      return -1;
  };

  if($self->{_debug}) {
      foreach(@devclasses) {
          ErmmTool::printtext('SYNCMSG1015', $_);
      }
  }

  DEVCLASSES: foreach my $devclass (@devclasses) {

      $self->_dsmcmd("query volume devclass=$devclass");
      foreach my $line ( @{$self->{_rawmsg}} ) {
          if ($line =~ /ANR2034E/) {
              # There are currently no volumes mounted according to TSM.
              next DEVCLASSES;

          } elsif ($line =~ /AN\w\d{4}\w .+/) {
              $$volumes = {};
              return -1;
          }

          my ($volume, $stgpool, $devclass,
              $capacity, $utilization) = split /,/, $line;
          $$volumes->{$volume} = { 'Type'        => 'volume',
                                   'StoragePool' => $stgpool,
                                   'Capacity'    => $capacity,
                                   'Utilization' => $utilization }
      }
  }

  return 0;
}

# Get all device classes that reference the library and return the
# devclasses names as list
# Return 0 on success and -1 on error.
sub devclasses {
    my($self, $devclasses) = @_;

    $self->_dsmcmd("query devclass format=detailed");
    foreach my $line (@{$self->{_rawmsg}}) {
          if ($line =~ /AN\w\d{4}\w .+/) {
              $$volumes = {};
              return -1;
          }
        my ($devclass, $strategy, $count, $type, $format, $capacity, $limit, $wait, $retention,
            $prefix, $devclass_library, $directory, $server, $retry) = split /,/, $line;

        if(defined $devclass_library && $devclass_library ne '') {
            if($self->{_debug}) {
                ErmmTool::printtext('SYNCMSG1016', $devclass, $devclass_library);
            }

            if ($self->{_libname} eq $devclass_library) {
                push @$devclasses, $devclass;
            }
        }
    }

    if (@$devclasses == 0) {
        (@{$self->{_rawmsg}}) = "Couldn't find TSM library $self->{_libname}.";
        return -1;
    }
    return 0;
}

# Get all drmedia (e.g. database backups) for one library.
#
# Since there is no limit option for the TSM command, this
# function gets all and then filters the result for the
# drmedia in the library.
#
# This should be ok, because there should not be much drmedias.
# Return 0 on success and -1 on error.
# We use the volume history as this won't start the ELM (unlike 'q drmedia')
sub drmedia {
    my ($self, $volumes) = @_;
    $self->_dsmcmd("query volhistory type=dbbackup");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if ($line =~ /ANR2034E/) {
            # There are currently no drmedia according to TSM.
            $$volumes = {};
            return 0;
        } elsif ($line =~ /AN\w\d{4}\w .+/) {
            $$volumes = {};
            return -1;
        # volume type, device class and volume name are needed
        } elsif ($line =~ /[^,]*,([^,]+),[^,]*,[^,]*,\".*\",([^,]+),([^,]+),.*,.*/) { #76755
            my $state = $1;
            my $deviceClass = $2;
            my $volume = $3;

            # We need to get the library of the volume (and we know
            # the volume exists). The match from volume to library is
            # done using the device class. A device class can only
            # have one library so use scalar.
            $self->_dsmcmd("query devc $deviceClass f=d");
            my ($devc) = @{$self->{_rawmsg}};
            my @devClassInfo = split /,/, $devc;
            my $library = $devClassInfo[10] if defined $devClassInfo[10];

            if ($self->{_libname} eq $library) {
                $$volumes->{$volume} = { 'Type'       => 'drmedia',
                                         'State'      => $state,
                                         'LastUpdate' => $lastupdate };
            }
        }
    }

    return 0;
} # end sub drmedia

# Get all snapshot volumes from the TSM server
# Return 0 on success and -1 on error
# We use the volume history as this won't start the ELM (unlike a SELECT ...)
sub snapshot {
    my ($self, $volumes) = @_;
    $self->_dsmcmd("query volhistory type=dbsnapshot");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if ($line =~ /ANR2034E/) {
            # There are currently no snapshot volumes according to TSM.
            $$volumes = {};
            return 0;
        } elsif ($line =~ /AN\w\d{4}\w .+/) {
            $$volumes = {};
            return -1;
        # volume type, device class and volume name are needed
        } elsif ($line =~ /[^,]*,([^,]+),[^,]*,[^,]*,\".*\",([^,]+),([^,]+),.*,.*/) { #76755

            my $state = $1;
            my $deviceClass = $2;
            my $volume = $3;

            # We need to get the library of the volume (and we know
            # the volume exists). The match from volume to library is
            # done using the device class. A device class can only
            # have one library so use scalar.
            $self->_dsmcmd("query devc $deviceClass f=d");
            my ($devc) = @{$self->{_rawmsg}};
            my @devClassInfo = split /,/, $devc;
            my $library = $devClassInfo[10] if defined $devClassInfo[10];

            if ($self->{_libname} eq $library) {
                $$volumes->{$volume} = { 'Type'       => 'dbsnapshot',
                                         'State'      => $state,
                                         'LastUpdate' => $lastupdate };
            }
        }
    }

    return 0;
} # end sub snapshot

sub backupset {
    my ($self, $volumes) = @_;
    $self->_dsmcmd("query volhistory type=backupset");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if ($line =~ /ANR2034E/) {
            # There are currently no backupset volumes according to TSM.
            $$volumes = {};
            return 0;
        } elsif ($line =~ /AN\w\d{4}\w .+/) {
            $$volumes = {};
            return -1;
        # volume type, device class and volume name are needed
        } elsif ($line =~ /[^,]*,([^,]+),[^,]*,[^,]*,[^,]*,([^,]+),([^,]+),[^,]*,.*/) { #76755

            my $state = $1;
            my $deviceClass = $2;
            my $volume = $3;

            # We need to get the library of the volume (and we know
            # the volume exists). The match from volume to library is
            # done using the device class. A device class can only
            # have one library so use scalar.
            $self->_dsmcmd("query devc $deviceClass f=d");
            my ($devc) = @{$self->{_rawmsg}};
            my @devClassInfo = split /,/, $devc;
            my $library = $devClassInfo[10] if defined $devClassInfo[10];

            if ($self->{_libname} eq $library) {
                $$volumes->{$volume} = { 'Type'       => 'backupset',
                                         'State'      => $state,
                                         'LastUpdate' => $lastupdate };
            }
        }
    }

    return 0;
} # end sub backupset

sub export {
    my ($self, $volumes) = @_;
    $self->_dsmcmd("query volhistory type=export");
    foreach my $line ( @{$self->{_rawmsg}} ) {
        if ($line =~ /ANR2034E/) {
            # There are currently no export volumes according to TSM.
            $$volumes = {};
            return 0;
        } elsif ($line =~ /AN\w\d{4}\w .+/) {
            $$volumes = {};
            return -1;
        # volume type, device class and volume name are needed
        } elsif ($line =~ /[^,]*,([^,]+),[^,]*,[^,]*,[^,]*,([^,]+),([^,]+),.*,.*/) { #76755
            my $state = $1;
            my $deviceClass = $2;
            my $volume = $3;

            # We need to get the library of the volume (and we know
            # the volume exists). The match from volume to library is
            # done using the device class. A device class can only
            # have one library so use scalar.
            $self->_dsmcmd("query devc $deviceClass f=d");
            my ($devc) = @{$self->{_rawmsg}};
            my @devClassInfo = split /,/, $devc;
            my $library = $devClassInfo[10] if defined $devClassInfo[10];

            if ($self->{_libname} eq $library) {
                $$volumes->{$volume} = { 'Type'       => 'export',
                                         'State'      => $state,
                                         'LastUpdate' => $lastupdate };
            }
        }
    }

    return 0;
} # end sub export

# Get all current mounts from the TSM server.
# Return 0 on success and -1 on error.
sub mounts {
  my ($self, $volumes) = @_;

  # There is no nice CSV format for this command, so the plan is to retrieve
  # the mounted volumes from  messages that look like this:
  # ANR8330I LTO volume 483AFQ is mounted R/W in drive TSLM
  $self->_dsmcmd("query mount");
  foreach my $line ( @{$self->{_rawmsg}} ) {

    if(defined && /^ANR8330I.*volume ([^ ]+) .*drive ([^ ]+) /) {
      my $volume  = $1;
      my $library = $2;

      if($library eq $self->{_libname} ) {
        $$volumes->{$volume} = { };
      }
    } elsif ($line =~ /ANR2034E/) {
        # There are currently no volumes mounted according to TSM.
       $$volumes = {};
        return 0;
    } elsif ($line =~ /AN\w\d{4}\w .+/) {
       $$volumes = {};
       return -1;
    }
  }

  return 0;
}

# Get last response.
sub getRawMsg {
    my ($self) = @_;
    return @{$self->{_rawmsg}};
}

# Issue TSM command using dsmadmc
sub _dsmcmd {
    my ($self, $dsmcmd) = @_;

    my $dsmflags='-commadelimited -dataonly=yes';

    my $command
        = $self->{_dsmadmc} . ' ' . $dsmflags
        . ' -server=' . $self->{_server}
        . ' -id=' . $self->{_id}
        . ' -password=' . $self->{_password}
        . ' ' . $dsmcmd;

  if($self->{_debug}) {
    ErmmTool::printtext('SYNCMSG1011', $command);
  }
#print "$query\n";
#print "$command\n";
  open my $query, "$command 2>&1 |" or die ErmmTool::printtext('SYNCMSG1014');
  my @result;
  while (<$query>) {
    chomp;
    push @result, $_;
    if($self->{_debug}) {
        ErmmTool::printtext('SYNCMSG1013', $_);
    }
  }
  close $query;
  $self->{_rawmsg} = \@result;
}

1;
